package java.awt;

import java.awt.geom.PathIterator;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import sun.awt.geom.Crossings;
import java.awt.geom.Rectangle2D;
import java.util.Arrays;
import java.io.Serializable;

public class Polygon implements Shape, Serializable
{
    public int npoints;
    public int[] xpoints;
    public int[] ypoints;
    protected Rectangle bounds;
    private static final long serialVersionUID = -6460061437900069969L;
    private static final int MIN_LENGTH = 4;
    
    public Polygon() {
        this.xpoints = new int[4];
        this.ypoints = new int[4];
    }
    
    public Polygon(int[] original, int[] original2, int newLength) {
        if (newLength > original.length || newLength > original2.length) {
            throw new IndexOutOfBoundsException("npoints > xpoints.length || npoints > ypoints.length");
        }
        if (newLength < 0) {
            throw new NegativeArraySizeException("npoints < 0");
        }
        this.npoints = newLength;
        this.xpoints = Arrays.copyOf(original, newLength);
        this.ypoints = Arrays.copyOf(original2, newLength);
        return;
    }
    
    public void reset() {
        this.npoints = 0;
        this.bounds = null;
    }
    
    public void invalidate() {
        this.bounds = null;
    }
    
    public void translate(int n, int n2) {
        for (int i = 0; i < this.npoints; ++i) {
            int[] xpoints = this.xpoints;
            int n3 = i;
            xpoints[n3] += n;
            int[] ypoints = this.ypoints;
            int n4 = i;
            ypoints[n4] += n2;
        }
        if (this.bounds != null) {
            this.bounds.translate(n, n2);
        }
    }
    
    void calculateBounds(int[] array, int[] array2, int n) {
        int min = Integer.MAX_VALUE;
        int min2 = Integer.MAX_VALUE;
        int max = Integer.MIN_VALUE;
        int max2 = Integer.MIN_VALUE;
        for (int i = 0; i < n; ++i) {
            int n2 = array[i];
            min = Math.min(min, n2);
            max = Math.max(max, n2);
            int n3 = array2[i];
            min2 = Math.min(min2, n3);
            max2 = Math.max(max2, n3);
        }
        this.bounds = new Rectangle(min, min2, max - min, max2 - min2);
    }
    
    void updateBounds(int x, int y) {
        if (x < this.bounds.x) {
            this.bounds.width += this.bounds.x - x;
            this.bounds.x = x;
        }
        else {
            this.bounds.width = Math.max(this.bounds.width, x - this.bounds.x);
        }
        if (y < this.bounds.y) {
            this.bounds.height += this.bounds.y - y;
            this.bounds.y = y;
        }
        else {
            this.bounds.height = Math.max(this.bounds.height, y - this.bounds.y);
        }
    }
    
    public void addPoint(int n, int n2) {
        if (this.npoints >= this.xpoints.length || this.npoints >= this.ypoints.length) {
            int highestOneBit = this.npoints * 2;
            if (highestOneBit < 4) {
                highestOneBit = 4;
            }
            else if ((highestOneBit & highestOneBit - 1) != 0x0) {
                highestOneBit = Integer.highestOneBit(highestOneBit);
            }
            this.xpoints = Arrays.copyOf(this.xpoints, highestOneBit);
            this.ypoints = Arrays.copyOf(this.ypoints, highestOneBit);
        }
        this.xpoints[this.npoints] = n;
        this.ypoints[this.npoints] = n2;
        ++this.npoints;
        if (this.bounds != null) {
            this.updateBounds(n, n2);
        }
    }
    
    @Override
    public Rectangle getBounds() {
        return this.getBoundingBox();
    }
    
    @Deprecated
    public Rectangle getBoundingBox() {
        if (this.npoints == 0) {
            return new Rectangle();
        }
        if (this.bounds == null) {
            this.calculateBounds(this.xpoints, this.ypoints, this.npoints);
        }
        return this.bounds.getBounds();
    }
    
    public boolean contains(Point point) {
        return this.contains(point.x, point.y);
    }
    
    public boolean contains(int n, int n2) {
        return this.contains((double)n, (double)n2);
    }
    
    @Deprecated
    public boolean inside(int n, int n2) {
        return this.contains((double)n, (double)n2);
    }
    
    @Override
    public Rectangle2D getBounds2D() {
        return this.getBounds();
    }
    
    @Override
    public boolean contains(double x, double y) {
        if (this.npoints <= 2 || !this.getBoundingBox().contains(x, y)) {
            return false;
        }
        int n = 0;
        int n2 = this.xpoints[this.npoints - 1];
        int n3 = this.ypoints[this.npoints - 1];
        int n4;
        int n5;
        for (int i = 0; i < this.npoints; n2 = n4, n3 = n5, ++i) {
            n4 = this.xpoints[i];
            n5 = this.ypoints[i];
            if (n5 != n3) {
                if (n4 < n2) {
                    if (x >= (double)n2) {
                        continue;
                    }
                }
                else {
                    if (x >= (double)n4) {
                        continue;
                    }
                    n4 = n2;
                }
                double n6;
                double n7;
                if (n5 < n3) {
                    if (y >= (double)n5) {
                        if (y >= (double)n3) {
                            continue;
                        }
                        if (x < (double)n4) {
                            ++n;
                            continue;
                        }
                        n6 = x - (double)n4;
                        n7 = y - (double)n5;
                    }
                    else {
                        continue;
                    }
                }
                else if (y >= (double)n3) {
                    if (y >= (double)n5) {
                        continue;
                    }
                    if (x < (double)n4) {
                        ++n;
                        continue;
                    }
                    n6 = x - (double)n2;
                    n7 = y - (double)n3;
                }
                else {
                    continue;
                }
                if (n6 < n7 / (double)(n3 - n5) * (double)(n2 - n4)) {
                    ++n;
                }
            }
        }
        return (n & 0x1) != 0x0;
    }
    
    private Crossings getCrossings(double xlo, double ylo, double xhi, double yhi) {
        Crossings.EvenOdd evenOdd = new Crossings.EvenOdd(xlo, ylo, xhi, yhi);
        int n = this.xpoints[this.npoints - 1];
        int n2 = this.ypoints[this.npoints - 1];
        int i = 0;
        while (i < this.npoints) {
            int n3 = this.xpoints[i];
            int n4 = this.ypoints[i];
            if (evenOdd.accumulateLine((double)n, (double)n2, (double)n3, (double)n4)) {
                return null;
            }
            n = n3;
            n2 = n4;
            ++i;
        }
        return evenOdd;
    }
    
    @Override
    public boolean contains(Point2D point2D) {
        return this.contains(point2D.getX(), point2D.getY());
    }
    
    @Override
    public boolean intersects(double x, double y, double w, double h) {
        if (this.npoints <= 0 || !this.getBoundingBox().intersects(x, y, w, h)) {
            return false;
        }
        Crossings crossings = this.getCrossings(x, y, x + w, y + h);
        return crossings == null || !crossings.isEmpty();
    }
    
    @Override
    public boolean intersects(Rectangle2D rectangle2D) {
        return this.intersects(rectangle2D.getX(), rectangle2D.getY(), rectangle2D.getWidth(), rectangle2D.getHeight());
    }
    
    @Override
    public boolean contains(double x, double y, double w, double h) {
        if (this.npoints <= 0 || !this.getBoundingBox().intersects(x, y, w, h)) {
            return false;
        }
        Crossings crossings = this.getCrossings(x, y, x + w, y + h);
        return crossings != null && crossings.covers(y, y + h);
    }
    
    @Override
    public boolean contains(Rectangle2D rectangle2D) {
        return this.contains(rectangle2D.getX(), rectangle2D.getY(), rectangle2D.getWidth(), rectangle2D.getHeight());
    }
    
    @Override
    public PathIterator getPathIterator(AffineTransform affineTransform) {
        return new Polygon.PolygonPathIterator(this, affineTransform);
    }
    
    @Override
    public PathIterator getPathIterator(AffineTransform affineTransform, double n) {
        return this.getPathIterator(affineTransform);
    }
    
    class PolygonPathIterator implements PathIterator
    {
        Polygon poly;
        AffineTransform transform;
        int index;
        
        public PolygonPathIterator(Polygon poly, AffineTransform transform) {
            this.poly = poly;
            this.transform = transform;
            if (poly.npoints == 0) {
                this.index = 1;
            }
        }
        
        @Override
        public int getWindingRule() {
            return 0;
        }
        
        @Override
        public boolean isDone() {
            return this.index > this.poly.npoints;
        }
        
        @Override
        public void next() {
            ++this.index;
        }
        
        @Override
        public int currentSegment(float[] array) {
            if (this.index >= this.poly.npoints) {
                return 4;
            }
            array[0] = (float)this.poly.xpoints[this.index];
            array[1] = (float)this.poly.ypoints[this.index];
            if (this.transform != null) {
                this.transform.transform(array, 0, array, 0, 1);
            }
            return this.index != 0?1:0;
        }
        
        @Override
        public int currentSegment(double[] array) {
            if (this.index >= this.poly.npoints) {
                return 4;
            }
            array[0] = (double)this.poly.xpoints[this.index];
            array[1] = (double)this.poly.ypoints[this.index];
            if (this.transform != null) {
                this.transform.transform(array, 0, array, 0, 1);
            }
            return this.index != 0?1:0;
        }
    }
}
